 //  CST340	or GT911

/* touch panel interface define */
sbit SDA	   =    P2^0;
sbit SCL       =    P2^1;
sbit PEN_INT       =    P2^2;	
sbit RESET	   =	P2^4;


#define uchar      unsigned char
#define uint       unsigned int
#define ulong      unsigned long


//Touch Status	 
#define Key_Down 0x01
#define Key_Up   0x00 

#define WHITE          0xFFFF
#define BLACK          0x0000
#define GRAY           0xF7DE
#define BLUE           0x001F
#define BLUE2          0x051F
#define RED            0xF800
#define PURPLE         0xF81F
#define GREEN          0x07E0
#define CYAN           0x7FFF
#define YELLOW         0xFFE0
#define DGREEN         0x07E0



struct _ts_event
{
    uint    x1;
    uint    y1;
    uint    x2;
    uint    y2;
    uint    x3;
    uint    y3;
    uint    x4;
    uint    y4;
    uint    x5;
    uint    y5;
	
    uchar     touch_point;
	uchar     Key_Sta;	
};

struct _ts_event ts_event; 

#define WRITE_ADD 0x28	//0X34  //	0xba
#define READ_ADD  0x29 	//0x35  //	0xbb 

#define CT_MAX_TOUCH    5		//ݴֵ֧ĵ

// Registers define
#define GTP_READ_COOR_ADDR  0x814e



void TOUCH_Init(void);
void TOUCH_Start(void);
void TOUCH_Stop(void);
uchar   TOUCH_Wait_Ack(void);
void TOUCH_Ack(void);
void TOUCH_NAck(void);

void TOUCH_Send_Byte(uchar txd);
uchar TOUCH_Read_Byte(unsigned char ack);
void Draw_Big_Point(uint x,uint y,uint colour);
uchar CST340_read_data(void);
void inttostr(uint value,uchar *str);


//IIC start
void TOUCH_Start(void)
{ 
	SDA=1;  
	_nop_();
	SCL=1;
	delay_us(3);
	SDA=0;  
	delay_us(3);
	SCL=0;
//	delay_us(1);
}	  


//IIC stop
void TOUCH_Stop(void)
{
	SDA=0;
	_nop_();
	SCL=1;
	delay_us(3);
	SDA=1;
	delay_us(3);
	SCL=0;
//	delay_us(1);						   	
}


//Wait for an answer signal
uchar TOUCH_Wait_Ack(void)
{	uchar errtime=0;

	SDA=1;
//	delay_us(1);
	SCL=1;
//	delay_us(1);
  	while(SDA)
	{
	    errtime++;
	    if(errtime>250)
		    {
		      TOUCH_Stop();
		      return 1;
		    }	
	
	   ;
	}
	SCL=0;
//	delay_us(1);
	return 0;
}



//Acknowledge
void TOUCH_Ack(void)
{	SCL=0;
	_nop_();
	SDA=0;
	delay_us(2);
	SCL=1;
	delay_us(2);
	SCL=0;
	_nop_();
}



//NO Acknowledge		    
void TOUCH_NAck(void)
{	SCL=0;
	delay_us(1);
	SDA=1;
	delay_us(2);
	SCL=1;
	delay_us(2);
	SCL=0;
	_nop_();
}	
	

//IIC send one byte		  
void TOUCH_Send_Byte(uchar Byte)
{	uchar t;  		
 	SCL=0; 
//	delay_us(5);
    for(t=0;t<8;t++)
    {           
	   	SDA=(bit)(Byte & 0x80) ;
	   	Byte <<=1;
		delay_us(2);
	   	SCL=1;
		delay_us(2);
	   	SCL=0;
	//	delay_us(5);
    }	

} 

//Read one byteack=0Send Acknowledgeack=1NO Acknowledge   
uchar TOUCH_Read_Byte(uchar ack)
{	uchar t,receive=0;

	SCL=0;
//	delay_us(5);
	SDA=1;
//	delay_us(5);
	for(t = 0; t < 8; t++)
	{	_nop_();
	 	SCL = 0;
		delay_us(4);
	 	SCL=1;
		delay_us(4);
	 	receive<<=1;
	 	if(SDA)receive=receive|0x01;
	//	delay_us(5);
	}

					 
   	if (ack)  TOUCH_NAck();//NO Acknowledge 
   	else       TOUCH_Ack(); //Send Acknowledge   
    
	 return receive;
}

uchar CST340_WR_Reg(uint reg,uchar *buf,uchar len)
{
	uchar i;
	uchar ret=0;
	TOUCH_Start();	
 	TOUCH_Send_Byte(WRITE_ADD);   	//д 	 
	TOUCH_Wait_Ack();
	TOUCH_Send_Byte(reg>>8);   	//͸8λַ
	TOUCH_Wait_Ack();	 										  		   
	TOUCH_Send_Byte(reg&0XFF);   	//͵8λַ
	TOUCH_Wait_Ack(); 
	for(i=0;i<len;i++)
	{	   
    	TOUCH_Send_Byte(buf[i]);  	//
		ret=TOUCH_Wait_Ack();
		if(ret)break;  
	}
    TOUCH_Stop();					//һֹͣ	    
	return ret; 
}


void CST340_RD_Reg(uint reg,uchar *buf,uchar len)
{
	uchar i;
 	TOUCH_Start();	
 	TOUCH_Send_Byte(WRITE_ADD);   	//д 	 
	TOUCH_Wait_Ack(); 
 	TOUCH_Send_Byte(reg>>8);   	//͸8λַ
	TOUCH_Wait_Ack();  										  		   
 	TOUCH_Send_Byte(reg&0XFF);   	//͵8λַ
	TOUCH_Wait_Ack();
  
 	TOUCH_Start(); 	 	   
	TOUCH_Send_Byte(READ_ADD);    //Ͷ		   
	TOUCH_Wait_Ack();    
	for(i=0;i<len;i++)
	{	   
    	buf[i]=TOUCH_Read_Byte(i==(len-1)?1:0); //
		
		
	//	if(i==(len-1))  buf[i]=TOUCH_Read_Byte(1);
	//	else            buf[i]=TOUCH_Read_Byte(0);	 
		
	} 
     TOUCH_Stop();//һֹͣ	    
}



uchar CST340_read_data(void)
{	uchar buf[50] = {0xff}; uchar ret = 0;	uchar ss[4];

  
	CST340_RD_Reg(GTP_READ_COOR_ADDR,buf,50);		

	ss[0]=0;		
 	CST340_WR_Reg(GTP_READ_COOR_ADDR,ss,1);	  //Must clear  Buffer status
 
  	ts_event.touch_point = buf[0] & 0x0f;



 	if (ts_event.touch_point == 0) 
		{  			

			return 0;
  		}		
   
 
					switch (ts_event.touch_point) 
					{
 						
							case 5:
				           			ts_event.x5 = (uint)(buf[35] )<<8 | (uint)buf[34];
				           			ts_event.y5 = (uint)(buf[37] )<<8 | (uint)buf[36];
				
						    case 4:
						           	ts_event.x4 = (uint)(buf[27] )<<8 | (uint)buf[26];
						           	ts_event.y4 = (uint)(buf[29] )<<8 | (uint)buf[28];
						
						    case 3:
						           	ts_event.x3 = (uint)(buf[19] )<<8 | (uint)buf[18];
						           	ts_event.y3 = (uint)(buf[21] )<<8 | (uint)buf[20];
						
						    case 2:
						           	ts_event.x2 = (uint)(buf[11] )<<8 | (uint)buf[10];
						           	ts_event.y2 = (uint)(buf[13] )<<8 | (uint)buf[12];
						
						    case 1:
						           	ts_event.x1 = (uint)(buf[3] )<<8 | (uint)buf[2];
						           	ts_event.y1 = (uint)(buf[5] )<<8 | (uint)buf[4];
				
				    break;
						    default:
						    return 0;
					}
					   




	   
	return ret;
}





void Pant(int dcolor)
{
	int i,j;
 	LCD_SetPos(0,0);//320x240
    for(i=0;i<320;i++)
	 {
	  for (j=0;j<240;j++)
	   	{
        	 Write_Data(dcolor>>8,dcolor);
	    }	  
	  }
}

void inttostr(int value,uchar *str)
{
	str[0]=value/1000+48;
	str[1]=value%1000/100+48;
	str[2]=value%1000%100/10+48;
 	str[3]=value%1000%100%10+48;
	str[4]=0;
}




//ָλʾһַ(8*12С)
//dcolorΪɫgbcolorΪɫ
void showzifu(unsigned int x,unsigned int y,unsigned char value,unsigned int dcolor,unsigned int bgcolor)	
{  
	unsigned char i,j;
	unsigned char *temp=zifu;    
    
	temp+=(value-32)*12;
	for(j=0;j<12;j++)
	{    LCD_SetPos(x,y); //
		for(i=0;i<8;i++)
		{ 		     
		 	if((*temp&(1<<(7-i)))!=0)
			{
				Write_Data(dcolor>>8,dcolor);
			} 
			else
			{
				Write_Data(bgcolor>>8,bgcolor);
			}   
		}
		y+=1;
		temp++;
	 }
}
//ָλʾһַ(8*12С)
//dcolorΪɫgbcolorΪɫ
void showzifustr(unsigned int x,unsigned int y,unsigned char *str,unsigned int dcolor,unsigned int bgcolor)	  
{  
	unsigned int x1,y1;
	x1=x;
	y1=y;
	while(*str!='\0')
	{	
		showzifu(x1,y1,*str,dcolor,bgcolor);
		x1+=7;
		str++;
	}	
}



void Draw_Big_Point(unsigned int x,unsigned int y,unsigned int colour)
{unsigned int k,h;	    
	
 		 
	   		for(k=0;k<2;k++)
			{
			    for(h=0;h<2;h++)
				{
					 LCD_SetPos(x+h,y+k);
			         Write_Data(colour>>8,colour);
				 }
			}
}


void TOUCH_Init(void)
{	
		RESET=0;
		PEN_INT=0;
		delayms(30);
 		PEN_INT=1;
		delayms(5);
		RESET=1;;
		delayms(20);
		ACC=PEN_INT;
		ACC=PEN_INT;
 		delayms(100);	 

 	/*	RESET=0;						   
		PEN_INT=0;
		delayms(20);
 		RESET=1;
		delayms(500);
		ACC=PEN_INT;
 		ACC=PEN_INT;
 		delayms(100); */
   
		ts_event.x1=0;
		ts_event.y1=0;
		ts_event.x2=0;
		ts_event.y2=0;
		ts_event.x3=0;
		ts_event.y3=0;
		ts_event.x4=0;
		ts_event.y4=0;
		ts_event.x5=0;
		ts_event.y5=0;

	   
}


void TPTEST(void)
{uchar ss[4]; uchar re=1;

   	ts_event.Key_Sta=Key_Up;	  
	 TOUCH_Init();	
 	Pant(0x0000);					
    showzifustr(80,18,"CTP TEST!",0,0xffff);	 
	while(KEY)
	{

 	  if(!PEN_INT) 
			{
			
					CST340_read_data();
	


					inttostr(ts_event.x1,ss);
					showzifustr(10,205,"X1:",BLUE,WHITE);
					showzifustr(35,205,ss,RED,WHITE);	
					inttostr(ts_event.y1,ss);
					showzifustr(80,205,"Y1:",BLUE,WHITE);
					showzifustr(105,205,ss,RED,WHITE);	
			
					Draw_Big_Point(ts_event.x1,ts_event.y1,RED);
					Draw_Big_Point(ts_event.x2,ts_event.y2,GREEN);	
					Draw_Big_Point(ts_event.x3,ts_event.y3,BLUE);
					Draw_Big_Point(ts_event.x4,ts_event.y4,CYAN);	
					Draw_Big_Point(ts_event.x5,ts_event.y5,PURPLE);
                   
			}



    }



}













